% Universidade de So Paulo (USP)
% Instituto de Cincias Matemticas e de Computao (ICMC)
% SCC0251 - Processamento de Imagens - 2013/1
% Prof. Dr. Mario Gazziro
% Monitor PAE: Vinicius Ruela Pereira Borges
%
% AULA 3: Segmentao de Imagens
% CDIGO: Mtodo de Otsu
% =========================================================

function limiar = otsu(I)

    %Histograma
    f = imhist(I);
    
    %altura e largura
    [M,N] = size(I);
    
    %Normaliza
    p = f/(M*N);
    
    %Quantidade de niveis de cinza
    L = 256;
    
    %Inicializao de limiar
    menor = 1e+20;
    
    for k = 2:L
        
        % Determina a distribuio
        q1 = 0; %Probabilidade do nivel de cinza ser da classe 1
        for i = 1:k
            q1 = q1 + p(i);
        end
        
        % q2 Satisfaz: q1 + q2 = 1
        q2 = 1 - q1; % Probabilidade do nivel de cinza ser da classe 2
        
        % Calculo da mdia para a classe 1
        m1 = 0;
        
        for i = 1:k
            ii = i-1;
            m1 = m1 + (ii*p(i));
        end
        m1 = m1/q1;
        
        % Calculo da mdia para a classe 2
        m2 = 0;
        
        for i = k:L
            ii = i-1;
            m2 = m2 + (ii*p(i));
        end
        m2 = m2/q2;
        
        % Calculo da varincia da classe 1
        s1 = 0;
        
        for i = 1:k
           ii = i-1;
           s1 = s1 + ((ii-m1)*(ii-m1)*(p(i)/q1));
        end
 
        % Calculo da varincia para a classe 2
        s2 = 0;
        
        for i = k:L
           ii = i-1;
           s2 = s2 + ((ii-m2)*(ii-m2)*(p(i)/q2));
        end

	% Calcula a varincia intra-classes
        sB = q1*s1 + q2*s2;
        
        % Verifica se eh menor do que sB calculada anteriormente
        if sB < menor
            limiar = k;
            menor = sB;
        end
    end
end
